Type: Package
Package: remstimate
Title: Optimization Frameworks for Tie-Oriented and Actor-Oriented
        Relational Event Models
Version: 3.0.0
Date: 2026-05-12
Authors@R: 
    c(person(given = "Giuseppe",
             family = "Arena",
             role = c("aut", "cre"),
             email = "g.arena@uva.nl",
             comment = c(ORCID = "0000-0001-5204-3326")),
      person(given = "Rumana",
             family = "Lakdawala",
             role = "aut"),
      person(given = "Fabio",
             family = "Generoso Vieira",
             role = "aut"),
      person(given = "Joris",
             family = "Mulder",
             role = "aut",
             email = "j.mulder3@tilburguniversity.edu"),
      person(given = "Marlyne",
             family = "Meijerink-Bosman",
             role = "ctb"),
      person(given = "Diana",
             family = "Karimova",
             role = "ctb"),
      person(given = "Mahdi",
             family = "Shafiee Kamalabad",
             role = "ctb"),
      person(given = "Roger",
             family = "Leenders",
             role = "ctb",
             email = "r.t.a.j.leenders@tilburguniversity.edu"))
Maintainer: Giuseppe Arena <g.arena@uva.nl>
Description: A comprehensive set of tools designed for optimizing likelihood within a tie-oriented (Butts, C., 2008, <doi:10.1111/j.1467-9531.2008.00203.x>) or an actor-oriented modelling framework (Stadtfeld, C., & Block, P., 2017, <doi:10.15195/v4.a14>) in relational event networks. The package accommodates both frequentist and Bayesian approaches. Maximum Likelihood Optimization (MLE) is supported. Bayesian estimation is done via Hamiltonian Monte Carlo (HMC).
License: MIT + file LICENSE
URL: https://tilburgnetworkgroup.github.io/remstimate/
BugReports: https://github.com/TilburgNetworkGroup/remstimate/issues
Depends: R (>= 4.0.0)
Imports: Rcpp, remify (>= 4.0.0), remstats (>= 4.0.0), trust, mvnfast
Suggests: knitr, rmarkdown, tinytest, survival
LinkingTo: Rcpp, RcppArmadillo, remify (>= 4.0.0)
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
LazyDataCompression: gzip
NeedsCompilation: yes
Packaged: 2026-05-12 15:31:37 UTC; jorismulder
Author: Giuseppe Arena [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5204-3326>),
  Rumana Lakdawala [aut],
  Fabio Generoso Vieira [aut],
  Joris Mulder [aut],
  Marlyne Meijerink-Bosman [ctb],
  Diana Karimova [ctb],
  Mahdi Shafiee Kamalabad [ctb],
  Roger Leenders [ctb]
Repository: CRAN
Date/Publication: 2026-05-13 05:10:20 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-13 09:57:15 UTC; unix
Archs: remstimate.so.dSYM
