Type: Package
Package: remstimate
Title: Optimization Frameworks for Tie-Oriented and Actor-Oriented
        Relational Event Models
Version: 3.1.0
Date: 2026-07-16
Authors@R: 
    c(person(given = "Giuseppe",
             family = "Arena",
             role = c("aut", "cre"),
             email = "g.arena@uva.nl",
             comment = c(ORCID = "0000-0001-5204-3326")),
      person(given = "Joris",
             family = "Mulder",
             role = "aut",
             email = "j.mulder3@tilburguniversity.edu"),
      person(given = "Rumana",
             family = "Lakdawala",
             role = "aut"),
      person(given = "Fabio",
             family = "Generoso Vieira",
             role = "aut"),
      person(given = "Marlyne",
             family = "Meijerink-Bosman",
             role = "ctb"),
      person(given = "Diana",
             family = "Karimova",
             role = "ctb"),
      person(given = "Mahdi",
             family = "Shafiee Kamalabad",
             role = "ctb"),
      person(given = "Roger",
             family = "Leenders",
             role = "ctb",
             email = "r.t.a.j.leenders@tilburguniversity.edu"))
Maintainer: Giuseppe Arena <g.arena@uva.nl>
Description: Tools for fitting, diagnosing, and analyzing tie-oriented and
    actor-oriented relational event models, under both frequentist and Bayesian
    approaches. The package supports tie-oriented modeling (Butts, 2008,
    <doi:10.1111/j.1467-9531.2008.00203.x>) and an actor-oriented modeling
    framework (Stadtfeld et al., 2017, <doi:10.15195/v4.a14>),
    with additional model diagnostics and goodness-of-fit tools. 
    Interfaces to estimation backends provide a range of extensions: random-effects (frailty)
    relational event models capturing sender, receiver, and dyadic heterogeneity
    (Juozaitiene & Wit 2024, <doi:10.1007/s11336-024-09952-x>;
    Mulder & Hoff, 2024, <doi:10.1214/24-AOAS1885>), finite mixture
    and dyadic latent class models for unobserved dyadic heterogeneity
    (Lakdawala et al., 2026, <doi:10.1016/j.socnet.2026.06.006>), penalized estimation via the lasso,
    ridge, and elastic net (Tibshirani, R., 1996,
    <doi:10.1111/j.2517-6161.1996.tb02080.x>; Karimova et al., 2023, <doi:10.1016/j.socnet.2023.02.006>),
    and approximate Bayesian regularization (Karimova et al., 2025, <doi:10.1016/j.jmp.2025.102925>). Modeling
    of events with a duration is also supported (Lakdawala et al., 2026, <doi:10.48550/arXiv.2602.21000>) and
    moving window relational event models (Mulder & Leenders, 2019,
    <doi:10.1016/j.chaos.2018.11.027>; Meijerink et al., 2023, <doi:10.1371/journal.pone.0272309>).
License: MIT + file LICENSE
URL: https://tilburgnetworkgroup.github.io/remstimate/
BugReports: https://github.com/TilburgNetworkGroup/remstimate/issues
Depends: R (>= 4.0.0), remify (>= 4.1.0), remstats (>= 4.1.0)
Imports: Rcpp, trust, mvnfast
Suggests: knitr, rmarkdown, tinytest, survival, coxme, glmnet, lme4,
        glmmTMB, flexmix, shrinkem (>= 0.4.0), MASS, nnet, remdata (>=
        0.2.1)
LinkingTo: Rcpp, RcppArmadillo, remify (>= 4.1.0)
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
LazyDataCompression: gzip
NeedsCompilation: yes
Packaged: 2026-07-17 08:33:20 UTC; jorismulder
Author: Giuseppe Arena [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5204-3326>),
  Joris Mulder [aut],
  Rumana Lakdawala [aut],
  Fabio Generoso Vieira [aut],
  Marlyne Meijerink-Bosman [ctb],
  Diana Karimova [ctb],
  Mahdi Shafiee Kamalabad [ctb],
  Roger Leenders [ctb]
Repository: CRAN
Date/Publication: 2026-07-17 16:20:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-17 16:50:44 UTC; unix
Archs: remstimate.so.dSYM
