# Generated by roxygen2: do not edit by hand

S3method(AIC,remstimate)
S3method(AICC,remstimate)
S3method(BIC,remstimate)
S3method(WAIC,remstimate)
S3method(bic_table,remstimate_mixrem_list)
S3method(coef,remstimate_durem)
S3method(coef,remstimate_glmm)
S3method(coef,remstimate_glmnet)
S3method(coef,remstimate_mixrem)
S3method(coef,remstimate_shrinkem)
S3method(coef,remstimate_window)
S3method(coef,remtribute)
S3method(diagnostics,remstimate)
S3method(diagnostics,remstimate_durem)
S3method(diagnostics,remstimate_glmm)
S3method(diagnostics,remstimate_glmnet)
S3method(diagnostics,remstimate_mixrem)
S3method(diagnostics,remstimate_shrinkem)
S3method(diagnostics,remstimate_window)
S3method(logLik,remstimate)
S3method(logLik,remstimate_durem)
S3method(logLik,remstimate_glmm)
S3method(logLik,remstimate_mixrem)
S3method(logLik,remtribute)
S3method(plot,diagnostics)
S3method(plot,diagnostics.remstimate_window)
S3method(plot,diagnostics_durem)
S3method(plot,remstimate)
S3method(plot,remstimate_durem)
S3method(plot,remstimate_glmnet)
S3method(plot,remstimate_mixrem_list)
S3method(plot,remstimate_shrinkem)
S3method(plot,remstimate_window)
S3method(print,diagnostics)
S3method(print,diagnostics.remstimate_window)
S3method(print,diagnostics_durem)
S3method(print,diagnostics_mixrem)
S3method(print,remstimate)
S3method(print,remstimate_durem)
S3method(print,remstimate_glmm)
S3method(print,remstimate_glmnet)
S3method(print,remstimate_mixrem)
S3method(print,remstimate_mixrem_list)
S3method(print,remstimate_shrinkem)
S3method(print,remstimate_window)
S3method(print,remtribute)
S3method(print,summary.remstimate_window)
S3method(summary,diagnostics)
S3method(summary,remstimate)
S3method(summary,remstimate_durem)
S3method(summary,remstimate_glmm)
S3method(summary,remstimate_glmnet)
S3method(summary,remstimate_mixrem)
S3method(summary,remstimate_shrinkem)
S3method(summary,remstimate_window)
S3method(summary,remtribute)
S3method(vcov,remtribute)
export(AICC)
export(WAIC)
export(bic_table)
export(diagnostics)
export(dlcrem)
export(frailty_rem)
export(remfrailty)
export(remixture)
export(rempenalty)
export(remstimate)
export(remtribute)
export(remwindow)
import(grDevices)
import(graphics)
import(remify)
import(remstats)
import(stats)
importFrom(Rcpp,sourceCpp)
importFrom(stats,AIC)
importFrom(stats,BIC)
useDynLib(remstimate, .registration = TRUE)
