#' @keywords internal
#' @docType package
#' @useDynLib remstimate, .registration = TRUE
#' @import stats grDevices graphics remify remstats
#' @importFrom Rcpp sourceCpp
#' @aliases remstimate-package
"_PACKAGE"