# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

remDerivatives <- function(pars, stats, actor1, actor2, dyad, omit_dyad, interevent_time, model, ordinal = FALSE, ncores = 1L, gradient = TRUE, hessian = TRUE, senderRate = TRUE, N = NULL, sender_riskset = NULL, receiver_riskset = NULL) {
    .Call(`_remstimate_remDerivatives`, pars, stats, actor1, actor2, dyad, omit_dyad, interevent_time, model, ordinal, ncores, gradient, hessian, senderRate, N, sender_riskset, receiver_riskset)
}

GDADAMAX <- function(pars, stats, actor1, actor2, dyad, omit_dyad, interevent_time, model, ordinal = FALSE, senderRate = TRUE, gradient = TRUE, hessian = FALSE, N = NULL, ncores = 1L, epochs = 1e03L, learning_rate = 0.002, beta1 = 0.9, beta2 = 0.999, epsilon = 0.01) {
    .Call(`_remstimate_GDADAMAX`, pars, stats, actor1, actor2, dyad, omit_dyad, interevent_time, model, ordinal, senderRate, gradient, hessian, N, ncores, epochs, learning_rate, beta1, beta2, epsilon)
}

HMC <- function(pars_init, nsim, nchains, burnin, meanPrior, sigmaPrior, stats, actor1, actor2, dyad, omit_dyad, interevent_time, model, ordinal = FALSE, ncores = 1L, senderRate = TRUE, N = NULL, thin = 1L, L = 100L, epsilon = 0.01, sender_riskset = NULL, receiver_riskset = NULL) {
    .Call(`_remstimate_HMC`, pars_init, nsim, nchains, burnin, meanPrior, sigmaPrior, stats, actor1, actor2, dyad, omit_dyad, interevent_time, model, ordinal, ncores, senderRate, N, thin, L, epsilon, sender_riskset, receiver_riskset)
}

getWAIC <- function(mu, vcov, pars, stats, actor1, actor2, dyad, interevent_time, omit_dyad, model, approach, ordinal = FALSE, ncores = 1L, senderRate = TRUE, nsim = 500L, sender_riskset = NULL, receiver_riskset = NULL) {
    .Call(`_remstimate_getWAIC`, mu, vcov, pars, stats, actor1, actor2, dyad, interevent_time, omit_dyad, model, approach, ordinal, ncores, senderRate, nsim, sender_riskset, receiver_riskset)
}

computeDiagnostics <- function(pars, stats, actor1, actor2, dyad, omit_dyad, model, N, senderRate = TRUE, ncores = 1L, baseline = 0) {
    .Call(`_remstimate_computeDiagnostics`, pars, stats, actor1, actor2, dyad, omit_dyad, model, N, senderRate, ncores, baseline)
}

remDerivativesSampled <- function(pars, stats, case_pos, samp_prob, interevent_time, ordinal = FALSE, gradient = TRUE, hessian = TRUE, ncores = 1L) {
    .Call(`_remstimate_remDerivativesSampled`, pars, stats, case_pos, samp_prob, interevent_time, ordinal, gradient, hessian, ncores)
}

