% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_ic.R
\name{print.remstimate}
\alias{print.remstimate}
\title{Print out a quick overview of a \code{remstimate} object}
\usage{
\method{print}{remstimate}(x, ...)
}
\arguments{
\item{x}{is a \code{remstimate} object.}

\item{...}{further arguments to be passed to the print method.}
}
\value{
no return value. Prints out the main characteristics of a 'remstimate' object.
}
\description{
A function that prints out the estimates returned by a 'remstimate' object.
}
\examples{

# ------------------------------------ #
#       method 'print' for the         #
# ------------------------------------ #

# loading data
data(ao_data)

# processing event sequence with remify
ao_reh <- remify::remify(edgelist = ao_data$edgelist, model = "actor")

# specifying linear predictor (for sender rate and receiver choice model)
rate_model <- ~ 1 + remstats::indegreeSender()
choice_model <- ~ remstats::inertia() + remstats::reciprocity()

# calculating statistics
ao_reh_stats <- remstats::remstats(reh = ao_reh,
                                   sender_effects = rate_model,
                                   receiver_effects = choice_model)

# running estimation
ao_mle <- remstimate::remstimate(reh = ao_reh,
                                 stats = ao_reh_stats,
                                 method = "MLE",
                                 nsim = 100,
                                 ncores = 1)

# print
ao_mle

# ------------------------------------ #
#   for more examples check vignettes  #
# ------------------------------------ #

}
